<?php

/**
 * ImagebamComHelper.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

/**
 * Class ImagebamComHelper
 *
 * A link like:
 * <a href="http://www.imagebam.com/image/9d9343518128940" target="_blank"><img src="http://thumbnails115.imagebam.com/51813/9d9343518128940.jpg" border="0" alt=""/></a>
 * takes me to:
 * http://www.imagebam.com/image/9d9343518128940
 * (see detail.imagebam.com.html for complete contents of that page)
 * which has this HTML:
 * <img class="image" id="i518128940" src="http://115.imagebam.com/download/ETLLOMFcv7vM9agrdfD9GA/51813/518128940/aria-giovanni-001.jpg" alt="loading"/>
 * So I need the src attribute to get the URI to the image.
 */
/**
 * this link in the thumbnail listing:
 * <a href="http://www.imagebam.com/image/7e365c874580814" target="_blank"><img src="http://thumbs2.imagebam.com/18/37/4d/7e365c874580814.jpg" border="0" alt="" /></a>>
 * takes me to
 * http://www.imagebam.com/image/7e365c874580814
 * which has this HTML
 * <a href="http://www.imagebam.com/image/7e365c874580814" title="Continue to your image" target="_self" style="font-size:30px;padding:15px;display: block;">Continue to your image</a>
 * which needs to be followed to get to
 * http://www.imagebam.com/image/7e365c874580814
 * which has
 * <img class="image" id="i874580814" src="http://images2.imagebam.com/0e/05/1d/7e365c874580814.jpg" alt="loading"/>
 */
class ImagebamComHelper extends BaseHelper
{
    /**
     * process
     * @param $uri
     * @return array Return array with 'image' and 'name' as the keys
     */
    public function process($uri) {
        $page = $this->getPage($uri);
//        $page = $this->getPage('test/detail.imagebam.com-3.html');
        $image['link'] = $this->getImageLink($page);
        $image['name'] = $this->getImageName($image['link']);
        $image['image'] = $this->getPage($image['link']);
        return $image;
    }
    
    public function getImageLink($html) {
        $metas = parse_array($html, '<meta', '/>');
        $link = false;
        foreach ($metas as $meta) {
            if (strstr($meta, 'og:image')) {
                $link = $this->getAttribute($meta, 'content');
            }
        }
        return $link;
    }

    public function getImageName($html) {
        $pieces = explode('/',$html);
        return array_pop($pieces);
    }

    /**
     * looking for the image info in a snippet like:
     * <div class="image-container">
     *  <img class="image" id="i518128940" src="http://115.imagebam.com/download/ETLLOMFcv7vM9agrdfD9GA/51813/518128940/aria-giovanni-001.jpg" alt="loading"/>
     * </div>
     * @param $html
     */
    public function getImageInfo($html) {
        $images = parse_array($html, '<img', '/>');
        foreach ($images as $image) {
            if (strstr($image, 'class="image"')) {
                $retval['link'] = $this->getAttribute($image, 'src');
                $retval['name'] = $this->getAttribute($image, 'id') . '.jpg';
            }
        }
        // $image['uri'] = get_attribute($images[0], 'src');
        // $parts = explode('/',$image['uri']);
        // $image['name'] = array_pop($parts);
        return $retval;
    }
}