<?php
/**
 * rip_config.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

if (!defined('PATH_TO_CONFIG')) {
    define('PATH_TO_CONFIG', 'conf/config.php');
}

/**
 * this function allows the user to update/edit their config file
 * @param $args
 * @param $rip
 */
function rip_config($args, $rip) {
    verbose('starting config', $args['verbose']);
    // check to see if we have an existing config file
    $exists = checkForExisting();

    $conf = [];
    if ($exists) {
        $action = getArgsExistingConf();

        if ($action == 'view') {
            displayCurrentConfig(loadConfig());
            exit;
        }
        if ($action == 'edit') {
            $conf = loadConfig();
        }
        if ($action == 'test_login') {
            displayTestLogin(loadConfig());
            exit;
        }

    }
    $confItems = confItems();

    $new = getArgsForConfig($confItems, $conf);

    $string = convertToString($confItems, $new);
    //backupConfigFile();
    $rs = writeConfFile(confTemplate($string));
    return $rs;
}

function checkForExisting() {
    if(file_exists(PATH_TO_CONFIG)) {
        return true;
    }
    return false;
}

function displayCurrentConfig($conf) {
    echo '----------------------'. PHP_EOL;
    echo 'Current Config Values'.PHP_EOL;
    foreach ($conf as $key => $value) {
        if ($value === true) {
            $value = 'true';
        }
        if ($value === false) {
            $value = 'false';
        }
        echo $key .' = '. $value . PHP_EOL;
    }
    echo '----------------------'. PHP_EOL;
    echo 'these values can be seen in: '. PATH_TO_CONFIG . PHP_EOL;
}

function displayTestLogin($conf) {
    verbose(__METHOD__ .' - attempting to login to Viper Girls with: '. $conf['username'], $conf['verbose']);
//    $cookie_before = file_get_contents('/Users/jackal/Sites/ripper/cookie-vipergirls.txt');
    $vg = new ViperGirls($conf['username'], $conf['password'], REQUESTING_USER_AGENT);
    $rs = $vg->loginToSite();
    // looking for the text "Thank you for logging in,"
    $pos = strpos($rs, 'Thank you for logging in,');
    if($pos === false) {
        echo_color('login FAILED', 'red');
    } else {
        echo_color('login succeeded', 'green');
    }
    echo PHP_EOL;
    exit;
}

/**
 * @return array $conf is the name of the array in the config file
 */
function loadConfig() {
    $conf = [];
    require PATH_TO_CONFIG;
    return $conf;
}

function getArgsForConfig($confItems, $current) {
    echo 'Configure' . PHP_EOL;
    echo 'For each of the following configurable items click Enter/Return to stay with the current value or enter a new value.'. PHP_EOL;
    echo '-----'. PHP_EOL;
    foreach ($confItems as $item) {
        if (array_key_exists($item['key'], $current)) {
            $item['default'] = $current[$item['key']];
        }
        echo $item['note'] . PHP_EOL;
        if (is_bool($item['default'])) {
            $value = $item['default'] ? 'true' : 'false';
        } else {
            $value = $item['default'];
        }
        echo $item['key'] .' = '. $value . PHP_EOL;
        $arg = trim(fgets(STDIN));
        if (empty($arg)) {
            $arg = $item['default'];
        }
        $retval[$item['key']] = $arg;
        echo PHP_EOL;
    }
    return $retval;
}


function writeConfFile($contents) {
    $rs = file_put_contents(PATH_TO_CONFIG, $contents);
    if ($rs) {
        echo_color('wrote new config file, values can be seen in: '. PATH_TO_CONFIG, 'green');
    }
    return $rs;
}

function backupConfigFile() {
    copy(PATH_TO_CONFIG, PATH_TO_CONFIG.'.bak');
}

/**
 * key is the name of the array key in the conf file
 * default is the default value for that key
 * note is the help text to display to the user
 * type is the kind of value, boolean, string, integer
 *
['key' => '', 'default' => '', 'note' => '', 'type'=>''],
 *
 * @return array
 */
function confItems() {
    $confItems = [
        ['key' => 'verbose', 'default' => 'false', 'note' => 'Set the verbose mode, which is a lot of information. Can be set to TRUE or FALSE. Default value is FALSE', 'type'=>'boolean'],
        ['key' => 'image_folder', 'default' => 'images/', 'note' => 'This is the path to the folder where you want to store the ripped images, must have a trailing slash. Default value is images/', 'type'=>'string'],
        ['key' => 'display_progress', 'default' => 'true', 'note' => 'Whether to display a progress indicator. Default value is TRUE', 'type'=>'boolean'],
        ['key' => 'display_progress_interval', 'default' => '10', 'note' => 'How often the progress indicator.  Default is 10', 'type'=>'integer'],
        ['key' => 'display_progress_character', 'default' => '.', 'note' => 'The character do you want to use as the progress indicator. Default value is . ', 'type'=>'string'],
        ['key' => 'image_set_threshold', 'default' => '5', 'note' => 'The minimum number of images needed to be considered an image set', 'type'=>'integer'],
        ['key' => 'prepend_increment', 'default' => 'false', 'note' => 'Some file hosts use string of numbers and letters like: 42a2dd1323232737.jpg for the filename.  If this is set to true we will add an incrementer to the filename so it looks something like: 01_42a2dd1323232737.jpg.  Default is FALSE', 'type'=>'boolean'],
        ['key' => 'append_increment', 'default' => 'false', 'note' => 'Instead of prepending, append the increment so the filename looks something like: 42a2dd1323232737_01.jpg.  Default is FALSE', 'type'=>'boolean'],
        ['key' => 'log_downloads', 'default' => 'true', 'note' => 'If set to TRUE all downloads will be logged into a SQLite database called laxaeripper.sqlite. If set to FALSE downloads will not be logged. Default is TRUE', 'type'=>'boolean'],
        ['key' => 'username', 'default' => 'username', 'note' => 'Your vipergirls.to username', 'type'=>'string'],
        ['key' => 'password', 'default' => 'password', 'note' => 'Your vipergirls.to password', 'type'=>'string'],
        ['key' => 'auto_like', 'default' => 'true', 'note' => 'When downloading an album, automatically \'Like\' the post.  Default is TRUE', 'type'=>'boolean'],
        ['key' => 'clean_title', 'default' => 'false', 'note' => 'Attempt to clean up the title, which gets used as the name of the image folder.  Default is TRUE', 'type'=>'boolean'],
    ];
    return $confItems;
}

function convertToString($confItems, $new) {
    $retval = '';
    foreach ($confItems as $item) {
        switch ($item['type']) {
        case 'boolean':
            if ($new[$item['key']] == 'true') {
                $value = 'true';
            } else {
                $value = 'false';
            }
            break;
        case 'integer':
            $value = (int)$new[$item['key']];
            break;

        case 'array':
            $value = convertToString();
            break;
        default:
            $value = "'". $new[$item['key']] ."'";
        }

        $retval .= '    \''. $item['key'] .'\' => '. $value .','. PHP_EOL;
    }
    return $retval;
}

function confTemplate($string) {
    $tpl = '<?php
/**
 * Please do not edit this file directly.  Instead use the Configure feature within the ripper.
 * This file was created by rip_config.php
 */
$conf = [
'.$string.'
];
';
    return $tpl;
}

function getArgsExistingConf() {
    echo PHP_EOL;
    echo 'You have a conf file would you like to edit it or replace it?'. PHP_EOL;
    echo '[ 1 ] to edit ' . PHP_EOL;
    echo '[ 2 ] to replace ' . PHP_EOL;
    echo '[ 3 ] view current config values' . PHP_EOL;
    echo '[ 9 ] test login' . PHP_EOL;
    echo 'Selection: ';
    $arg = trim(fgets(STDIN));
    if (empty($arg)) {
        $arg = 3;
    }
    switch ($arg) {
        case "1":
            $retval = 'edit';
            break;
        case "2":
            $retval = 'replace';
            break;
        case "3":
            $retval = 'view';
            break;
        case "9":
            $retval = 'test_login';
            break;
        default:
            $retval = 'view';
    }
    return $retval;
}