<?php
/**
 * rip_thread.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2020
 */

function rip_thread($args, $rip) {
    verbose(__METHOD__ .' - starting rip_thread', $rip->getVerbose());

    $rs = $rip->loginToSite($args['uri']);
    if ($rs) {
        $logged_in = 'true';
    } else {
        $logged_in = 'false';
    }
    verbose(__METHOD__ .' logged in is: '. $logged_in, $rip->getVerbose());

    verbose(__METHOD__ .' - grabbing page: '. $args['uri'], $rip->getVerbose());

    /**
     * opts
     * is an array that can affect how the file is downloaded, not built yet though
     */
    $opts = null;

    $log = $rip->getConfigValue('log_downloads');
    $like = $rip->getConfigValue('auto_like');
    $clean = $rip->getConfigValue('clean_title');

    $xmlstring = $rip->getDownloader()->getXml($rip->getDownloader()->getApiUri($args['uri'], $opts));
//    out($xmlstring);
    $xml = simplexml_load_string($xmlstring);
    $contents = getThreadInfoFromXml($xml);
    /**
     * we may have more than one post on the thread, if we do, ask user which post(s) to download
     */
    if (count($contents['posts']) > 1) {
        $posts_to_rip = getArgsSelectPost($contents['posts']);
        if (strtolower($posts_to_rip) == strtolower('all')) {
            $sets = range(1, count($contents['posts']));
        } else {
            $sets = explode(',', $posts_to_rip);
        }
    } else {
        $sets = [1];
    }

    verbose('processing: '. count($sets) .' set(s)', $args['verbose']);
    foreach ($sets as $set) {
        $post_id = $contents['posts'][$set-1]['attributes']['id'];
        $post = getPostFromXml($xml, $post_id);

        // title might be available in $args['title'] if we are using rip_albums_in_file
        if ($clean) {
            $title = getArgsTitle(cleanUpTitle($post['attributes']['title']));
        } else {
            $title = getArgsTitle($post['attributes']['title']);
        }

        $start = getArgsStart();
        if ($start !== 0) {
            $rip->setStart($start);
        }

        $rip->makeImagesSaveToFolder($title);

        $rs = $rip->processXml($post['images']);
        if (!$rs) {
            $retval[] = $rip->getFails();
        }
        // log the download, if log_downloads is true
        if ($log) {
            $log_rs = $rip->db->insert([$title, $args['uri'], $rip->getProgressCounter()]);
        }
        // add a like to post, if auto_like is true
        if ($like) {
            doLikeOfPost($post_id,$contents['hash'], $rip->getDownloader());
        }

        $retval[] = $rs;
        $rip->resetCounter();
    }
    return $retval;
}


function getArgsSelectPost($posts) {
    echo PHP_EOL;
    echo 'Download which Post(s)? '. PHP_EOL;
    $counter = 1;
    foreach ($posts as $post) {
        $host = getImageHost($post['first_image']);
        if (!$host) {
            $host = 'UNKNOWN HOST';
        }
        echo '['. $counter .'] '. $post['attributes']['title'] .' - '. $host . PHP_EOL;
        $counter++;
    }
    echo 'Enter the set numbers you want to rip, separated by commas.  Enter ALL for all sets.';
    echo PHP_EOL;
    echo 'Which post(s) to rip? ';
    $arg = trim(fgets(STDIN));

    if (!$arg) {
        $arg = 1;
    }
    return $arg;
}

function getArgsStart() {
    echo 'start after image #: ';
    $start = trim(fgets(STDIN));

    if ($start === '') {
        $start = 0;
    }
    return $start;
}
